use crate::error::Result;
use pyo3::PyAny;

pub trait PyAnyExtras<'py> {
    fn qualified_class_name(&self) -> Result<String>;
    fn get_proto_meta(&self) -> Result<&'py PyAny>;
    fn create_instance(&self, field_name: &str) -> Result<&'py PyAny>;
    fn is_list_field(&self, field_name: &str) -> Result<bool>;
}

impl<'py> PyAnyExtras<'py> for &'py PyAny {
    fn qualified_class_name(&self) -> Result<String> {
        let class = self.getattr("__class__")?;
        let module = class.getattr("__module__")?;
        let name = class.getattr("__name__")?;
        Ok(format!("{module}.{name}"))
    }

    fn get_proto_meta(&self) -> Result<&'py PyAny> {
        Ok(self.getattr("_betterproto")?)
    }

    fn create_instance(&self, field_name: &str) -> Result<&'py PyAny> {
        let res = self
            .getattr("cls_by_field")?
            .get_item(field_name)?
            .call0()?;
        Ok(res)
    }

    fn is_list_field(&self, field_name: &str) -> Result<bool> {
        let cls = self.getattr("default_gen")?.get_item(field_name)?;
        let module = cls.getattr("__module__")?;
        let name = cls.getattr("__name__")?;
        Ok(module.to_string() == "builtins" && name.to_string() == "list")
    }
}
